diff --git a/ld/ldlang.c b/ld/ldlang.c
index 54f160c4db..cd24b3838b 100644
--- a/ld/ldlang.c
+++ b/ld/ldlang.c
@@ -3418,6 +3418,12 @@ ldlang_add_undef (const char *const name, bfd_boolean cmdline)
     insert_undefined (new_undef->name);
 }
 
+/* Determine if a bfd is MIPS based. Stolen from bfd/elfxx-mips.c */
+#define is_mips_elf(bfd)                                \
+  (bfd_get_flavour (bfd) == bfd_target_elf_flavour      \
+   && elf_tdata (bfd) != NULL                           \
+   && elf_object_id (bfd) == MIPS_ELF_DATA)
+
 /* Insert NAME as undefined in the symbol table.  */
 
 static void
@@ -3432,6 +3438,12 @@ insert_undefined (const char *name)
     {
       h->type = bfd_link_hash_undefined;
       h->u.undef.abfd = NULL;
+
+      if (link_info.output_bfd != NULL &&
+              is_mips_elf(link_info.output_bfd) &&
+              is_elf_hash_table (link_info.hash))
+          ((struct elf_link_hash_entry *)h)->mark = 1;
+
       bfd_link_add_undef (link_info.hash, h);
     }
 }
